/*
 *  hwinfo.c
 *  
 *
 *  Created by Marco Bambini on 8/29/08.
 *  Copyright 2005-2011 SQLabs, Inc. All rights reserved.
 *
 */

/*
	SHOW HARDWARE INFO
 
	A very simple custom command that returns a cursor with a key, value table filled with hardware information from the current machine.
 
 */

#include "hwinfo.h"

#define PLUGIN_NAME			"SHOW HARDWARE INFO"
#define PLUGIN_VERSION		"1.0"
#define PLUGIN_COPYRIGHT	"(c) 2011 SQLabs"
#define PLUGIN_DESCRIPTION	"This is a sample custom command plugin that adds the SHOW HARDWARE INFO command to the server."

void cubesql_plugin_init (void)
{
	// Nothing to allocate/prepare at startup
	return;
}

void cubesql_plugin_exit (void)
{
	// Nothing to free on exit
	return;
}

void cubesql_plugin_info (char **name, char **version, char **copyright, char **description)
{	
	// Returns plugins informatio
	SET_PLUGIN_NAME(PLUGIN_NAME);
	SET_PLUGIN_VERSION(PLUGIN_VERSION);
	SET_PLUGIN_COPYRIGHT(PLUGIN_COPYRIGHT);
	SET_PLUGIN_DESCRIPTION(PLUGIN_DESCRIPTION);
}

void cubesql_plugin_register (void *pluginref)
{
	cubesql_register_command (pluginref, "SHOW HARDWARE INFO", kTRUE, kFALSE, kSELECT, kCommandIDNone, privilege_None, show_hardware_info);
}

int show_hardware_info (cubesql_client *client, const char **args, int commandID)
{
	cubesql_db	*db = NULL;
	char		sql[1024];
	#ifdef CUBESQL_WINDOWS
	SYSTEM_INFO siSysInfo;
	#else
	char		p[512];
	size_t		len;
	#endif
	#ifdef CUBESQL_LINUX
	int			mib[2];
	#endif
	
	// create in-memory database
	db = cubesql_db_create_inmemory();
	if (db == NULL) {
		cubesql_client_set_error(client, 1, "Unable to create in-memory database.");
		goto abort_plugin;
	}
	
	// create Info table
	cubesql_db_write(db, "CREATE TABLE Info (key TEXT, value TEXT);");
	
	// Core plugin's function
	#ifdef CUBESQL_MAC
	len = 512;
	sysctlbyname("hw.machine", p, &len, NULL, 0);
	snprintf(sql, sizeof(sql), "INSERT INTO Info (key, value) VALUES ('MACHINE', '%s');", p);
	cubesql_db_write(db, sql);
	
	len = 512;
	sysctlbyname("hw.model", p, &len, NULL, 0);
	snprintf(sql, sizeof(sql), "INSERT INTO Info (key, value) VALUES ('MODEL', '%s');", p);
	cubesql_db_write(db, sql);
	#endif
	
	#ifdef CUBESQL_WINDOWS
	GetSystemInfo(&siSysInfo);
	
	_snprintf(sql, sizeof(sql), "INSERT INTO Info (key, value) VALUES ('OEM_ID', '%u');", siSysInfo.dwOemId);
	cubesql_db_write(db, sql);

	_snprintf(sql, sizeof(sql), "INSERT INTO Info (key, value) VALUES ('PROCESSOR_TYPE', '%u');", siSysInfo.dwProcessorType); 
	cubesql_db_write(db, sql);
	#endif

	#ifdef CUBESQL_LINUX
	len = 512;
	mib[0] = CTL_HW;
	mib[1] = HW_MACHINE;
	sysctl(mib, 2, p, &len, NULL, 0);
	snprintf(sql, sizeof(sql), "INSERT INTO Info (key, value) VALUES ('MACHINE', '%s');", p);
	cubesql_db_write(db, sql);
	
	len = 512;
	mib[0] = CTL_HW;
	mib[1] = HW_MODEL;
	sysctl(mib, 2, p, &len, NULL, 0);
	snprintf(sql, sizeof(sql), "INSERT INTO Info (key, value) VALUES ('MODEL', '%s');", p);
	cubesql_db_write(db, sql);
	#endif
	
	cubesql_db_send_cursor(db, "SELECT * FROM Info;", client);
	cubesql_db_close(db);
	return CUBESQL_SEND_CURSOR;
	
abort_plugin:
	if (db) cubesql_db_close(db);
	return CUBESQL_SEND_ERROR;
}
